@@ -3,70 +3,99 @@ import re
 
 class Idcard_Indonesia(): 
     def __init__(self): 
-        a = None 
+        self.NAME_FLAG = ["Nama", "Nam", "Iama", "Lama", "lama", "Vama", "vama"] 
+        self.GENDER_FLAG = ["PEREMPUAN", " PEREMPUAN", "EREMPUAN", "PEREMPUA", "FEMALE"]
+        self.KEY = []
+
 
     def __get_ic(self, text_list): 
         ic_number = "-"
+        get_ic = False
 
         for i in text_list: 
             if re.match(r'^([\s\d]+)$', i): 
                 ic_number = i 
+                get_ic = True
                 break
-        return ic_number
+        return ic_number, get_ic
 
     
     def __get_name(self, text_list): 
         name = "-"
+        get_name = False
+
         for idx, i in enumerate(text_list): 
-            if i in ["Nama"]: 
+            if i in self.NAME_FLAG: 
                 name = text_list[idx + 1]
-                break
-        return name 
+                if bool(re.match('[A-Z ]+$', name)) \
+                    and not(bool(re.search(r'\d', name))): 
+                    get_name = True
+                    break
+                else: 
+                    name = text_list[idx + 2]
+                    if (bool(re.match('[A-Z ]+$', name))) \
+                        and not(bool(re.search(r'\d', name))): 
+                        get_name = True
+                        break
+                    else: 
+                        name = text_list[max(0, idx - 1)]
+                        if (bool(re.match('[A-Z ]+$', name)))\
+                             and not(bool(re.search(r'\d', name))): 
+                            get_name = True
+                            break
+                        else: 
+                            name = "-"
+                    
+        return name, get_name
 
     
     def __get_dob(self, text_list): 
         dob = "-"
-        for idx, i in enumerate(text_list): 
-            if "Tempa" in i: 
-                dob = text_list[idx + 1]
-                break 
+        get_dob = False
 
-        return dob 
+        for idx, i in enumerate(text_list): 
+            pos_dob = re.findall(" \d{1,2}-\d{1,2}-\d{4}", i)
+            if len(pos_dob) > 0: 
+                dob = pos_dob[0]
+                dob = dob.replace(" ", "")
+                get_dob = True
+
+        return dob, get_dob
 
     
     def __get_address(self, text_list): 
         address = "-"
-        for idx, i in enumerate(text_list): 
-            if i in ["Alamat"]: 
-                address = text_list[idx + 7]
-                break
-            elif i in ["Kecamatan"]: 
-                address = text_list[idx + 1]
-                break 
-        
-        address = address + " " + text_list[1] + " " + text_list[0]
-        return address
+        get_address = False 
+
+        if ("KAB" in text_list[1] or "OTA" in text_list[1]) and "PROV" in text_list[0]: 
+            get_address = True
+            address = text_list[1] + " " + text_list[0]
+        return address, get_address
 
 
     def __get_gender(self, text_list): 
         gender = "-"
+        get_gender = False
+
         for idx, i in enumerate(text_list): 
-            if i in ["Jenis kelamin"]: 
-                gender = text_list[idx + 1]
-                break 
-        if "LAKI" in gender or "LAK" in gender or "AKI" in gender: 
-            gender = "M"
-        else: 
-            gender = "F"
-        return gender 
+            if bool(re.match('[A-Z- ]+$', i)): 
+                if bool(re.match("[A-Z ]{2,4}I-L[A-Z ]{2,4}", i)) or i =="MALE":
+                    gender = "M"
+                    get_gender = True
+                    break
+                elif i in self.GENDER_FLAG: 
+                    gender = "F"
+                    get_gender = True
+                    break
+        return gender, get_gender
 
 
     def process(self, text_list): 
-        name = self.__get_name(text_list)
-        num_ic = self.__get_ic(text_list)
-        dob = self.__get_dob(text_list)
-        gender = self.__get_gender(text_list)
-        address = self.__get_address(text_list)
+        name, get_name = self.__get_name(text_list)
+        num_ic, get_ic = self.__get_ic(text_list)
+        dob, get_dob = self.__get_dob(text_list)
+        gender, get_gender = self.__get_gender(text_list)
+        address, get_address = self.__get_address(text_list)
         state = "-"
 
         result = {"ic_number": num_ic, "name": name, 
@@ -90,7 +119,7 @@ class Idcard_Malaysia():
 
 
     def __get_ic(self, text_list): 
-        selected_ic = None
+        selected_ic = '-'
         ic_index = None
         get = False
 
@@ -115,7 +144,7 @@ class Idcard_Malaysia():
         else:
             actualyear = actualyear + 2000
 
-        return day+'/'+month+'/'+ str(actualyear)
+        return day+'-'+month+'-'+ str(actualyear)
 
 
     def __get_gender(self, ic_number):
@@ -168,12 +197,14 @@ class Idcard_Malaysia():
     
     def process(self, text_list): 
         ic_number, ic_index, get_ic = self.__get_ic(text_list)
-        full_name = None 
-        gender = None 
-        dob = None 
-        address = None 
+        full_name = '-' 
+        gender = '-'
+        dob = '-'
+        address = '-'
+        state = '-'
 
         if get_ic: 
+            print (ic_number)
             full_name = self.__get_fullname(text_list, ic_index)
             print (full_name)
             gender = self.__get_gender(ic_number)
@@ -182,7 +213,6 @@ class Idcard_Malaysia():
             print (dob)
             address, state = self.__get_address(text_list)
             print (address)
-            print (ic_number)
         
         result = {"ic_number": ic_number, "name": full_name, 
                   "gender": gender, "dob": dob, 
@@ -208,11 +238,11 @@ class Idcard_Singapore():
                 name = text_list[idx + 1]
             if "IDENTITY" in i or "CARD NO" in i: 
                 num_ic = i.split(" ")[-1]
-            if "Date" in i: 
+            if "Date" in i or "birth" in i: 
                 dob = text_list[idx + 2] 
                 gender = text_list[idx + 3]
                 if gender != "M" and gender != "F": 
-                    gender = "unknown"
+                    gender = "-"
 
         print (name)
         print (num_ic) 
@@ -240,14 +270,14 @@ class Idcard_Vietnam():
         state = "-"
 
         for idx, i in enumerate(text_list): 
-            if  "Ho ten" in i: 
+            if  "Ho ten" in i or "Ho te" in i or "o ten" in i: 
                 name = text_list[idx + 1]
 
             for j in self.DOB_CHARS: 
                 if j in i: 
                     dob = text_list[idx + 1]
 
-            if "Nguyen quan" in i: 
+            if "Nguyen quan" in i or "quan" in i or "Nguyen " in i: 
                 address = text_list[idx + 1] + ", " + text_list[idx + 2]
 
         print (name)
@@ -276,17 +306,17 @@ class Idcard_Thailand():
         state = "-"
 
         for idx, i in enumerate(text_list): 
-            if "Name" in i: 
+            if "Name" in i or "Nam" in i or "ame" in i: 
                 name = i.split(" ")[-1]
-                if "Miss" in i or "Mrs" in i: 
+                if "Miss" in i or "Mrs" in i or "Mis" in i: 
                     gender = 'F'
                 elif "Mr" in i: 
                     gender = "M"
 
-            if "Last name" in i: 
+            if "Last name" in i or "Last nam" in i or "ast nam" in i: 
                 name = name + " " + i.split(" ")[-1]
             
-            if "Date of Birth" in i: 
+            if "Date of Birth" in i or "date o" in i or " birth" in i: 
                 dob = i.split(" ")[-3] + " " + i.split(" ")[-2] + i.split(" ")[-1]
                 dob = dob.replace('.', "")
 
